package com.yandex.appmetrica_plugin;

import android.app.Activity;
import android.content.Context;
import android.location.Location;
import android.os.Handler;
import android.os.Looper;
import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import com.yandex.android.metrica.flutter.pigeon.Pigeon;
import com.yandex.metrica.AppMetricaDeviceIDListener;
import com.yandex.metrica.DeferredDeeplinkListener;
import com.yandex.metrica.DeferredDeeplinkParametersListener;
import com.yandex.metrica.YandexMetrica;
import com.yandex.metrica.ecommerce.ECommerceEvent;
import com.yandex.metrica.plugins.PluginErrorDetails;
import java.util.HashMap;
import java.util.Map;

public class AppMetricaImpl implements Pigeon.AppMetricaPigeon {

    @NonNull
    private final Context context;
    @NonNull
    private final Handler mainHandler = new Handler(Looper.getMainLooper());

    @Nullable
    public Activity activity = null;

    public AppMetricaImpl(
        @NonNull final Context context
    ) {
        this.context = context;
    }

    @Override
    public void activate(@NonNull Pigeon.AppMetricaConfigPigeon config) {
        YandexMetrica.activate(context, Converter.toNative(config));
    }

    @Override
    public void activateReporter(@NonNull Pigeon.ReporterConfigPigeon config) {
        YandexMetrica.activateReporter(context, Converter.toNative(config));
    }

    @Override
    public void touchReporter(@NonNull String apiKey) {
        YandexMetrica.getReporter(context, apiKey);
    }

    @NonNull
    @Override
    public Long getLibraryApiLevel() {
        return (long) YandexMetrica.getLibraryApiLevel();
    }

    @NonNull
    @Override
    public String getLibraryVersion() {
        return YandexMetrica.getLibraryVersion();
    }

    @Override
    public void resumeSession() {
        YandexMetrica.resumeSession(activity);
    }

    @Override
    public void pauseSession() {
        YandexMetrica.pauseSession(activity);
    }

    @Override
    public void reportAppOpen(@NonNull String deeplink) {
        YandexMetrica.reportAppOpen(deeplink);
    }

    @Override
    public void reportError(@NonNull Pigeon.ErrorDetailsPigeon error, @Nullable String message) {
        YandexMetrica.getPluginExtension().reportError(Converter.toNative(error), message);
    }

    @Override
    public void reportErrorWithGroup(@NonNull String groupId, @Nullable Pigeon.ErrorDetailsPigeon error, @Nullable String message) {
        final PluginErrorDetails pluginErrorDetails;
        if (error != null) {
            pluginErrorDetails = Converter.toNative(error);
        } else {
            pluginErrorDetails = null;
        }
        YandexMetrica.getPluginExtension().reportError(groupId, message, pluginErrorDetails);
    }

    @Override
    public void reportUnhandledException(@NonNull Pigeon.ErrorDetailsPigeon error) {
        YandexMetrica.getPluginExtension().reportUnhandledException(Converter.toNative(error));
    }

    @Override
    public void reportEventWithJson(@NonNull String eventName, @Nullable String attributesJson) {
        YandexMetrica.reportEvent(eventName, attributesJson);
    }

    @Override
    public void reportEvent(@NonNull String eventName) {
        YandexMetrica.reportEvent(eventName);
    }

    @Override
    public void reportReferralUrl(@NonNull String referralUrl) {
        YandexMetrica.reportReferralUrl(referralUrl);
    }

    @Override
    public void requestDeferredDeeplink(@NonNull Pigeon.Result<Pigeon.AppMetricaDeferredDeeplinkPigeon> result) {
        YandexMetrica.requestDeferredDeeplink(new DeferredDeeplinkListener() {
            @Override
            public void onDeeplinkLoaded(@NonNull String deeplink) {
                mainHandler.post(() -> result.success(new Pigeon.AppMetricaDeferredDeeplinkPigeon.Builder()
                    .setDeeplink(deeplink)
                    .build()
                ));
            }

            @Override
            public void onError(@NonNull DeferredDeeplinkListener.Error error, @Nullable String messageArg) {
                final Pigeon.AppMetricaDeferredDeeplinkReasonPigeon errorPigeon;
                switch (error) {
                    case NOT_A_FIRST_LAUNCH:
                        errorPigeon = Pigeon.AppMetricaDeferredDeeplinkReasonPigeon.NOT_A_FIRST_LAUNCH;
                        break;
                    case PARSE_ERROR:
                        errorPigeon = Pigeon.AppMetricaDeferredDeeplinkReasonPigeon.PARSE_ERROR;
                        break;
                    case UNKNOWN:
                        errorPigeon = Pigeon.AppMetricaDeferredDeeplinkReasonPigeon.UNKNOWN;
                        break;
                    case NO_REFERRER:
                        errorPigeon = Pigeon.AppMetricaDeferredDeeplinkReasonPigeon.NO_REFERRER;
                        break;
                    default:
                        errorPigeon = Pigeon.AppMetricaDeferredDeeplinkReasonPigeon.UNKNOWN;
                        break;
                }
                mainHandler.post(() -> result.success(new Pigeon.AppMetricaDeferredDeeplinkPigeon.Builder()
                    .setDeeplink(null)
                    .setError(new Pigeon.AppMetricaDeferredDeeplinkErrorPigeon.Builder()
                        .setReason(errorPigeon)
                        .setMessage(messageArg)
                        .setDescription(error.getDescription())
                        .build()
                    )
                    .build()
                ));
            }
        });
    }

    @Override
    public void requestDeferredDeeplinkParameters(@NonNull Pigeon.Result<Pigeon.AppMetricaDeferredDeeplinkParametersPigeon> result) {
        YandexMetrica.requestDeferredDeeplinkParameters(new DeferredDeeplinkParametersListener() {
            @Override
            public void onParametersLoaded(@NonNull Map<String, String> params) {
                mainHandler.post(() -> result.success(new Pigeon.AppMetricaDeferredDeeplinkParametersPigeon.Builder()
                    .setParameters(new HashMap<>(params))
                    .build()
                ));
            }

            @Override
            public void onError(@NonNull DeferredDeeplinkParametersListener.Error error, @NonNull String messageArg) {
                final Pigeon.AppMetricaDeferredDeeplinkReasonPigeon errorPigeon;
                switch (error) {
                    case NOT_A_FIRST_LAUNCH:
                        errorPigeon = Pigeon.AppMetricaDeferredDeeplinkReasonPigeon.NOT_A_FIRST_LAUNCH;
                        break;
                    case PARSE_ERROR:
                        errorPigeon = Pigeon.AppMetricaDeferredDeeplinkReasonPigeon.PARSE_ERROR;
                        break;
                    case UNKNOWN:
                        errorPigeon = Pigeon.AppMetricaDeferredDeeplinkReasonPigeon.UNKNOWN;
                        break;
                    case NO_REFERRER:
                        errorPigeon = Pigeon.AppMetricaDeferredDeeplinkReasonPigeon.NO_REFERRER;
                        break;
                    default:
                        errorPigeon = Pigeon.AppMetricaDeferredDeeplinkReasonPigeon.UNKNOWN;
                        break;
                }
                mainHandler.post(() -> result.success(new Pigeon.AppMetricaDeferredDeeplinkParametersPigeon.Builder()
                    .setParameters(null)
                    .setError(new Pigeon.AppMetricaDeferredDeeplinkErrorPigeon.Builder()
                        .setReason(errorPigeon)
                        .setMessage(messageArg)
                        .setDescription(error.getDescription())
                        .build()
                    )
                    .build()
                ));
            }
        });
    }

    @Override
    public void requestAppMetricaDeviceID(@NonNull Pigeon.Result<Pigeon.AppMetricaDeviceIdPigeon> result) {
        YandexMetrica.requestAppMetricaDeviceID(new AppMetricaDeviceIDListener() {
            @Override
            public void onLoaded(@Nullable String deviceIdResult) {
                mainHandler.post(() -> result.success(new Pigeon.AppMetricaDeviceIdPigeon.Builder()
                    .setDeviceId(deviceIdResult)
                    .setErrorReason(Pigeon.AppMetricaDeviceIdReasonPigeon.NO_ERROR)
                    .build()
                ));
            }

            @Override
            public void onError(@NonNull AppMetricaDeviceIDListener.Reason reason) {
                final Pigeon.AppMetricaDeviceIdReasonPigeon errorPigeon;
                switch (reason) {
                    case UNKNOWN:
                        errorPigeon = Pigeon.AppMetricaDeviceIdReasonPigeon.UNKNOWN;
                        break;
                    case INVALID_RESPONSE:
                        errorPigeon = Pigeon.AppMetricaDeviceIdReasonPigeon.INVALID_RESPONSE;
                        break;
                    case NETWORK:
                        errorPigeon = Pigeon.AppMetricaDeviceIdReasonPigeon.NETWORK;
                        break;
                    default:
                        errorPigeon = Pigeon.AppMetricaDeviceIdReasonPigeon.UNKNOWN;
                        break;
                }
                mainHandler.post(() -> result.success(new Pigeon.AppMetricaDeviceIdPigeon.Builder()
                    .setErrorReason(errorPigeon)
                    .build()
                ));
            }
        });
    }

    @Override
    public void sendEventsBuffer() {
        YandexMetrica.sendEventsBuffer();
    }

    @Override
    public void setLocation(@Nullable Pigeon.LocationPigeon location) {
        final Location nativeLocation;
        if (location != null) {
            nativeLocation = Converter.toNative(location);
        } else {
            nativeLocation = null;
        }
        YandexMetrica.setLocation(nativeLocation);
    }

    @Override
    public void setLocationTracking(@NonNull Boolean enabled) {
        YandexMetrica.setLocationTracking(enabled);
    }

    @Override
    public void setStatisticsSending(@NonNull Boolean enabled) {
        YandexMetrica.setStatisticsSending(context, enabled);
    }

    @Override
    public void setUserProfileID(@Nullable String userProfileID) {
        YandexMetrica.setUserProfileID(userProfileID);
    }

    @Override
    public void reportUserProfile(@NonNull Pigeon.UserProfilePigeon userProfile) {
        YandexMetrica.reportUserProfile(Converter.toNative(userProfile));
    }

    @Override
    public void putErrorEnvironmentValue(@NonNull String key, @Nullable String value) {
        YandexMetrica.putErrorEnvironmentValue(key, value);
    }

    @Override
    public void reportRevenue(@NonNull Pigeon.RevenuePigeon revenue) {
        YandexMetrica.reportRevenue(Converter.toNative(revenue));
    }

    @Override
    public void reportECommerce(@NonNull Pigeon.ECommerceEventPigeon event) {
        final ECommerceEvent eCommerceEvent = ECommerceConverter.toNative(event);
        if (eCommerceEvent != null) {
            YandexMetrica.reportECommerce(eCommerceEvent);
        }
    }

    @Override
    public void handlePluginInitFinished() {
        YandexMetrica.resumeSession(activity);
    }

    @Override
    public void reportAdRevenue(@NonNull Pigeon.AdRevenuePigeon adRevenue) {
        YandexMetrica.reportAdRevenue(Converter.toNative(adRevenue));
    }
}
