package com.yandex.appmetrica_plugin;

import android.location.Location;
import androidx.annotation.NonNull;
import com.yandex.android.metrica.flutter.pigeon.Pigeon;
import com.yandex.metrica.AdRevenue;
import com.yandex.metrica.AdType;
import com.yandex.metrica.PreloadInfo;
import com.yandex.metrica.ReporterConfig;
import com.yandex.metrica.Revenue;
import com.yandex.metrica.YandexMetricaConfig;
import com.yandex.metrica.plugins.PluginErrorDetails;
import com.yandex.metrica.plugins.StackTraceItem;
import com.yandex.metrica.profile.Attribute;
import com.yandex.metrica.profile.BirthDateAttribute;
import com.yandex.metrica.profile.BooleanAttribute;
import com.yandex.metrica.profile.CounterAttribute;
import com.yandex.metrica.profile.GenderAttribute;
import com.yandex.metrica.profile.NameAttribute;
import com.yandex.metrica.profile.NotificationsEnabledAttribute;
import com.yandex.metrica.profile.NumberAttribute;
import com.yandex.metrica.profile.StringAttribute;
import com.yandex.metrica.profile.UserProfile;
import com.yandex.metrica.profile.UserProfileUpdate;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Currency;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class Converter {

    @NonNull
    public static Revenue toNative(@NonNull final Pigeon.RevenuePigeon pigeon) {
        final Revenue.Builder resultBuilder = Revenue.newBuilderWithMicros(
            new BigDecimal(pigeon.getPrice()).multiply(new BigDecimal(1_000_000)).longValue(),
            Currency.getInstance(pigeon.getCurrency())
        );

        final String productId = pigeon.getProductId();
        if (productId != null) {
            resultBuilder.withProductID(productId);
        }

        final String payload = pigeon.getPayload();
        if (payload != null) {
            resultBuilder.withPayload(payload);
        }

        final Long quantity = pigeon.getQuantity();
        if (quantity != null) {
            resultBuilder.withQuantity(quantity.intValue());
        }

        final Pigeon.ReceiptPigeon receiptPigeon = pigeon.getReceipt();
        if (receiptPigeon != null) {
            final Revenue.Receipt.Builder receiptBuilder = Revenue.Receipt.newBuilder();

            final String data = receiptPigeon.getData();
            if (data != null) {
                receiptBuilder.withData(data);
            }

            final String signature = receiptPigeon.getSignature();
            if (signature != null) {
                receiptBuilder.withSignature(signature);
            }

            resultBuilder.withReceipt(receiptBuilder.build());
        }
        return resultBuilder.build();
    }

    @NonNull
    public static YandexMetricaConfig toNative(@NonNull final Pigeon.AppMetricaConfigPigeon pigeon) {
        final YandexMetricaConfig.Builder resultBuilder = YandexMetricaConfig.newConfigBuilder(pigeon.getApiKey());

        final String appVersion = pigeon.getAppVersion();
        if (appVersion != null) {
            resultBuilder.withAppVersion(appVersion);
        }

        final Boolean crashReporting = pigeon.getCrashReporting();
        if (crashReporting != null) {
            resultBuilder.withCrashReporting(crashReporting);
        }

        final Boolean firstActivationAsUpdate = pigeon.getFirstActivationAsUpdate();
        if (firstActivationAsUpdate != null) {
            resultBuilder.handleFirstActivationAsUpdate(firstActivationAsUpdate);
        }

        final Pigeon.LocationPigeon location = pigeon.getLocation();
        if (location != null) {
            resultBuilder.withLocation(Converter.toNative(location));
        }

        final Boolean locationTracking = pigeon.getLocationTracking();
        if (locationTracking != null) {
            resultBuilder.withLocationTracking(locationTracking);
        }

        final Boolean logs = pigeon.getLogs();
        if (logs != null) {
            resultBuilder.withLogs();
        }

        final Long maxReportsInDatabaseCount = pigeon.getMaxReportsInDatabaseCount();
        if (maxReportsInDatabaseCount != null) {
            resultBuilder.withMaxReportsInDatabaseCount(maxReportsInDatabaseCount.intValue());
        }

        final Boolean nativeCrashReporting = pigeon.getNativeCrashReporting();
        if (nativeCrashReporting != null) {
            resultBuilder.withNativeCrashReporting(nativeCrashReporting);
        }

        final Long sessionTimeout = pigeon.getSessionTimeout();
        if (sessionTimeout != null) {
            resultBuilder.withSessionTimeout(sessionTimeout.intValue());
        }

        final Boolean sessionsAutoTracking = pigeon.getSessionsAutoTracking();
        if (sessionsAutoTracking != null) {
            resultBuilder.withSessionsAutoTrackingEnabled(sessionsAutoTracking);
        }

        final Boolean statisticsSending = pigeon.getStatisticsSending();
        if (statisticsSending != null) {
            resultBuilder.withStatisticsSending(statisticsSending);
        }

        final Pigeon.PreloadInfoPigeon preloadInfoPigeon = pigeon.getPreloadInfo();
        if (preloadInfoPigeon != null) {
            final PreloadInfo.Builder preloadInfoBuilder = PreloadInfo.newBuilder(preloadInfoPigeon.getTrackingId());

            final Map<Object, Object> additionalInfo = preloadInfoPigeon.getAdditionalInfo();
            if (additionalInfo != null) {
                for(Map.Entry<Object, Object> entry : additionalInfo.entrySet()) {
                    preloadInfoBuilder.setAdditionalParams((String) entry.getKey(), (String) entry.getValue());
                }
            }
            resultBuilder.withPreloadInfo(preloadInfoBuilder.build());
        }

        final Map<String, String> errorEnvironment = pigeon.getErrorEnvironment();
        if (errorEnvironment != null) {
            for(Map.Entry<String, String> entry : errorEnvironment.entrySet()) {
                resultBuilder.withErrorEnvironmentValue(entry.getKey(), entry.getValue());
            }
        }

        final String userProfileID = pigeon.getUserProfileID();
        if (userProfileID != null) {
            resultBuilder.withUserProfileID(userProfileID);
        }

        final Boolean revenueAutoTracking = pigeon.getRevenueAutoTracking();
        if (revenueAutoTracking != null) {
            resultBuilder.withRevenueAutoTrackingEnabled(revenueAutoTracking);
        }

        final Boolean appOpenTrackingEnabled = pigeon.getAppOpenTrackingEnabled();
        if (appOpenTrackingEnabled != null) {
            resultBuilder.withAppOpenTrackingEnabled(appOpenTrackingEnabled);
        }

        return resultBuilder.build();
    }

    @NonNull
    public static ReporterConfig toNative(@NonNull final Pigeon.ReporterConfigPigeon pigeon) {
        final ReporterConfig.Builder resultBuilder = ReporterConfig.newConfigBuilder(pigeon.getApiKey());

        final Boolean logs = pigeon.getLogs();
        if (logs != null) {
            resultBuilder.withLogs();
        }

        final Long maxReportsInDatabaseCount = pigeon.getMaxReportsInDatabaseCount();
        if (maxReportsInDatabaseCount != null) {
            resultBuilder.withMaxReportsInDatabaseCount(maxReportsInDatabaseCount.intValue());
        }

        final Long sessionTimeout = pigeon.getSessionTimeout();
        if (sessionTimeout != null) {
            resultBuilder.withSessionTimeout(sessionTimeout.intValue());
        }

        final Boolean statisticsSending = pigeon.getStatisticsSending();
        if (statisticsSending != null) {
            resultBuilder.withStatisticsSending(statisticsSending);
        }

        final String userProfileID = pigeon.getUserProfileID();
        if (userProfileID != null) {
            resultBuilder.withUserProfileID(userProfileID);
        }

        return resultBuilder.build();
    }

    @NonNull
    public static Location toNative(@NonNull final Pigeon.LocationPigeon pigeon) {
        final Location result = new Location(pigeon.getProvider());
        result.setLongitude(pigeon.getLongitude());
        result.setLatitude(pigeon.getLatitude());

        final Double altitude = pigeon.getAltitude();
        if (altitude != null) {
            result.setAltitude(altitude);
        }

        final Double course = pigeon.getCourse();
        if (course != null) {
            result.setBearing(course.floatValue());
        }

        final Long timestamp = pigeon.getTimestamp();
        if (timestamp != null) {
            result.setTime(timestamp);
        }

        final Double accuracy = pigeon.getAccuracy();
        if (accuracy != null) {
            result.setAccuracy(accuracy.floatValue());
        }

        final Double speed = pigeon.getSpeed();
        if (speed != null) {
            result.setSpeed(speed.floatValue());
        }

        return result;
    }

    private static final Map<Pigeon.GenderPigeon, GenderAttribute.Gender> genderToNative =
        new HashMap<Pigeon.GenderPigeon, GenderAttribute.Gender>() {{
            put(Pigeon.GenderPigeon.MALE, GenderAttribute.Gender.MALE);
            put(Pigeon.GenderPigeon.FEMALE, GenderAttribute.Gender.FEMALE);
            put(Pigeon.GenderPigeon.OTHER, GenderAttribute.Gender.OTHER);
    }};

    @NonNull
    public static UserProfile toNative(@NonNull final Pigeon.UserProfilePigeon pigeon) {
        final UserProfile.Builder resultBuilder = UserProfile.newBuilder();

        final List<Pigeon.UserProfileAttributePigeon> attributePigeons = pigeon.getAttributes();

        final List<UserProfileUpdate<?>> userProfileUpdates = new ArrayList<>();
        for (final Pigeon.UserProfileAttributePigeon attributePigeon: attributePigeons) {
            final Pigeon.UserProfileAttributeType type = attributePigeon.getType();
            if (type != null) {
                switch (type) {
                    case BIRTH_DATE: {
                        final BirthDateAttribute attribute = Attribute.birthDate();

                        if (Boolean.TRUE.equals(attributePigeon.getReset())) {
                            userProfileUpdates.add(attribute.withValueReset());
                        } else {
                            final Long year = attributePigeon.getYear();
                            final Long month = attributePigeon.getMonth();
                            final Long day = attributePigeon.getDay();
                            final Long age = attributePigeon.getAge();

                            if (year == null) {
                                if (age != null) {
                                    userProfileUpdates.add(attribute.withAge(age.intValue()));
                                }
                            } else {
                                if (month == null) {
                                    userProfileUpdates.add(attribute.withBirthDate(year.intValue()));
                                } else {
                                    if (day == null) {
                                        userProfileUpdates.add(attribute.withBirthDate(
                                            year.intValue(),
                                            month.intValue()
                                        ));
                                    } else {
                                        userProfileUpdates.add(attribute.withBirthDate(
                                            year.intValue(),
                                            month.intValue(),
                                            day.intValue()
                                        ));
                                    }
                                }
                            }
                        }
                        break;
                    }

                    case BOOLEAN: {
                        final BooleanAttribute attribute = Attribute.customBoolean(attributePigeon.getKey());
                        if (Boolean.TRUE.equals(attributePigeon.getReset())) {
                            userProfileUpdates.add(attribute.withValueReset());
                        } else {
                            final Boolean boolValue = attributePigeon.getBoolValue();
                            final boolean boolValueWithDefault = boolValue != null ? boolValue : false;
                            if (Boolean.TRUE.equals(attributePigeon.getIfUndefined())) {
                                userProfileUpdates.add(attribute.withValueIfUndefined(boolValueWithDefault));
                            } else {
                                userProfileUpdates.add(attribute.withValue(boolValueWithDefault));
                            }
                        }
                        break;
                    }

                    case COUNTER: {
                        final CounterAttribute attribute = Attribute.customCounter(attributePigeon.getKey());

                        final Double doubleValue = attributePigeon.getDoubleValue();
                        final double doubleValueWithDefault = doubleValue != null ? doubleValue : 0.0;
                        userProfileUpdates.add(attribute.withDelta(doubleValueWithDefault));
                        break;
                    }

                    case GENDER: {
                        final GenderAttribute attribute = Attribute.gender();
                        if (Boolean.TRUE.equals(attributePigeon.getReset())) {
                            userProfileUpdates.add(attribute.withValueReset());
                        } else {
                            final GenderAttribute.Gender nativeGender = genderToNative.get(attributePigeon.getGenderValue());
                            final GenderAttribute.Gender nativeGenderWithDefault = nativeGender != null ? nativeGender : GenderAttribute.Gender.OTHER;
                            userProfileUpdates.add(attribute.withValue(nativeGenderWithDefault));
                        }
                        break;
                    }

                    case NAME: {
                        final NameAttribute attribute = Attribute.name();
                        if (Boolean.TRUE.equals(attributePigeon.getReset())) {
                            userProfileUpdates.add(attribute.withValueReset());
                        } else {
                            final String stringValue = attributePigeon.getStringValue();
                            final String stringValueWithDefault = stringValue != null ? stringValue : "";
                            userProfileUpdates.add(attribute.withValue(stringValueWithDefault));
                        }
                        break;
                    }

                    case NOTIFICATION_ENABLED: {
                        final NotificationsEnabledAttribute attribute = Attribute.notificationsEnabled();
                        if (Boolean.TRUE.equals(attributePigeon.getReset())) {
                            userProfileUpdates.add(attribute.withValueReset());
                        } else {
                            final Boolean boolValue = attributePigeon.getBoolValue();
                            final boolean boolValueWithDefault = boolValue != null ? boolValue : false;
                            userProfileUpdates.add(attribute.withValue(boolValueWithDefault));
                        }
                        break;
                    }

                    case NUMBER: {
                        final NumberAttribute attribute = Attribute.customNumber(attributePigeon.getKey());
                        if (Boolean.TRUE.equals(attributePigeon.getReset())) {
                            userProfileUpdates.add(attribute.withValueReset());
                        } else {
                            final Double doubleValue = attributePigeon.getDoubleValue();
                            final double doubleValueWithDefault = doubleValue != null ? doubleValue : 0.0;
                            if (Boolean.TRUE.equals(attributePigeon.getIfUndefined())) {
                                userProfileUpdates.add(attribute.withValueIfUndefined(doubleValueWithDefault));
                            } else {
                                userProfileUpdates.add(attribute.withValue(doubleValueWithDefault));
                            }
                        }
                        break;
                    }

                    case STRING: {
                        final StringAttribute attribute = Attribute.customString(attributePigeon.getKey());
                        if (Boolean.TRUE.equals(attributePigeon.getReset())) {
                            userProfileUpdates.add(attribute.withValueReset());
                        } else {
                            final String stringValue = attributePigeon.getStringValue();
                            final String stringValueWithDefault = stringValue != null ? stringValue : "";
                            if (Boolean.TRUE.equals(attributePigeon.getIfUndefined())) {
                                userProfileUpdates.add(attribute.withValueIfUndefined(stringValueWithDefault));
                            } else {
                                userProfileUpdates.add(attribute.withValue(stringValueWithDefault));
                            }
                        }
                        break;
                    }
                }
            }
        }

        for (final UserProfileUpdate<?> userProfileUpdate: userProfileUpdates) {
            if (userProfileUpdate != null) {
                resultBuilder.apply(userProfileUpdate);
            }
        }

        return resultBuilder.build();
    }

    @NonNull
    public static StackTraceItem toNative(@NonNull final Pigeon.StackTraceElementPigeon pigeon) {
        return new StackTraceItem.Builder()
            .withFileName(pigeon.getFileName())
            .withClassName(pigeon.getClassName())
            .withMethodName(pigeon.getMethodName())
            .withLine(pigeon.getLine().intValue())
            .withColumn(pigeon.getColumn().intValue())
            .build();
    }

    @NonNull
    public static PluginErrorDetails toNative(@NonNull final Pigeon.ErrorDetailsPigeon pigeon) {
        final PluginErrorDetails.Builder resultBuilder = new PluginErrorDetails.Builder()
            .withExceptionClass(pigeon.getExceptionClass())
            .withMessage(pigeon.getMessage())
            .withPlatform(PluginErrorDetails.Platform.FLUTTER)
            .withVirtualMachineVersion(pigeon.getDartVersion());

        final List<Pigeon.StackTraceElementPigeon> stackTraceElementPigeons = pigeon.getBacktrace();
        if (stackTraceElementPigeons != null) {
            final List<StackTraceItem> stackTraceElements = new ArrayList<>();
            for (final Pigeon.StackTraceElementPigeon stackTraceElementPigeon : stackTraceElementPigeons) {
                stackTraceElements.add(Converter.toNative(stackTraceElementPigeon));
            }
            resultBuilder.withStacktrace(stackTraceElements);
        }
        return resultBuilder.build();
    }

    private static final Map<Pigeon.AdTypePigeon, AdType> adTypeToNative =
        new HashMap<Pigeon.AdTypePigeon, AdType>() {{
            put(Pigeon.AdTypePigeon.UNKNOWN, null);
            put(Pigeon.AdTypePigeon.NATIVE, AdType.NATIVE);
            put(Pigeon.AdTypePigeon.BANNER, AdType.BANNER);
            put(Pigeon.AdTypePigeon.REWARDED, AdType.REWARDED);
            put(Pigeon.AdTypePigeon.INTERSTITIAL, AdType.INTERSTITIAL);
            put(Pigeon.AdTypePigeon.MREC, AdType.MREC);
            put(Pigeon.AdTypePigeon.OTHER, AdType.OTHER);
    }};

    @NonNull
    public static AdRevenue toNative(@NonNull final Pigeon.AdRevenuePigeon pigeon) {
        final AdRevenue.Builder resultBuilder = AdRevenue.newBuilder(
            new BigDecimal(pigeon.getAdRevenue()),
            Currency.getInstance(pigeon.getCurrency())
        );

        final Pigeon.AdTypePigeon adTypePigeon = pigeon.getAdType();
        if (adTypePigeon != null) {
            final AdType adType = adTypeToNative.get(adTypePigeon);
            if (adType != null) {
                resultBuilder.withAdType(adType);
            }
        }

        final String adNetwork = pigeon.getAdNetwork();
        if (adNetwork != null) {
            resultBuilder.withAdNetwork(adNetwork);
        }

        final String adUnitId = pigeon.getAdUnitId();
        if (adUnitId != null) {
            resultBuilder.withAdUnitId(adUnitId);
        }

        final String adUnitName = pigeon.getAdUnitName();
        if (adUnitName != null) {
            resultBuilder.withAdUnitName(adUnitName);
        }

        final String adPlacementId = pigeon.getAdPlacementId();
        if (adPlacementId != null) {
            resultBuilder.withAdPlacementId(adPlacementId);
        }

        final String adPlacementName = pigeon.getAdPlacementName();
        if (adPlacementName != null) {
            resultBuilder.withAdPlacementName(adPlacementName);
        }

        final String precision = pigeon.getPrecision();
        if (precision != null) {
            resultBuilder.withPrecision(precision);
        }

        final Map<String, String> payload = pigeon.getPayload();
        if (payload != null) {
            resultBuilder.withPayload(payload);
        }

        return resultBuilder.build();
    }
}
