package com.yandex.appmetrica_plugin;

import android.content.Context;
import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import com.yandex.android.metrica.flutter.pigeon.Pigeon;
import com.yandex.metrica.YandexMetrica;
import com.yandex.metrica.ecommerce.ECommerceEvent;
import com.yandex.metrica.plugins.PluginErrorDetails;

public class ReporterImpl implements Pigeon.ReporterPigeon {

    @NonNull
    final Context context;

    ReporterImpl(@NonNull final Context context) {
        this.context = context;
    }

    @Override
    public void sendEventsBuffer(@NonNull String apiKey) {
        YandexMetrica.getReporter(context, apiKey).sendEventsBuffer();
    }

    @Override
    public void reportEvent(@NonNull String apiKey, @NonNull String eventName) {
        YandexMetrica.getReporter(context, apiKey).reportEvent(eventName);
    }

    @Override
    public void reportEventWithJson(@NonNull String apiKey, @NonNull String eventName, @Nullable String attributesJson) {
        YandexMetrica.getReporter(context, apiKey).reportEvent(eventName, attributesJson);
    }

    @Override
    public void reportError(@NonNull String apiKey, @NonNull Pigeon.ErrorDetailsPigeon error, @Nullable String message) {
        YandexMetrica.getReporter(context, apiKey).getPluginExtension()
            .reportError(Converter.toNative(error), message);
    }

    @Override
    public void reportErrorWithGroup(
        @NonNull String apiKey,
        @NonNull String groupId,
        @Nullable Pigeon.ErrorDetailsPigeon error,
        @Nullable String message
    ) {
        final PluginErrorDetails pluginErrorDetails;
        if (error != null) {
            pluginErrorDetails = Converter.toNative(error);
        } else {
            pluginErrorDetails = null;
        }
        YandexMetrica.getReporter(context, apiKey).getPluginExtension()
            .reportError(groupId, message, pluginErrorDetails);
    }

    @Override
    public void reportUnhandledException(@NonNull String apiKey, @NonNull Pigeon.ErrorDetailsPigeon error) {
        YandexMetrica.getReporter(context, apiKey).getPluginExtension()
            .reportUnhandledException(Converter.toNative(error));
    }

    @Override
    public void resumeSession(@NonNull String apiKey) {
        YandexMetrica.getReporter(context, apiKey).resumeSession();
    }

    @Override
    public void pauseSession(@NonNull String apiKey) {
        YandexMetrica.getReporter(context, apiKey).pauseSession();
    }

    @Override
    public void setStatisticsSending(@NonNull String apiKey, @NonNull Boolean enabled) {
        YandexMetrica.getReporter(context, apiKey).setStatisticsSending(enabled);
    }

    @Override
    public void setUserProfileID(@NonNull String apiKey, @Nullable String userProfileID) {
        YandexMetrica.getReporter(context, apiKey).setUserProfileID(userProfileID);
    }

    @Override
    public void reportUserProfile(@NonNull String apiKey, @NonNull Pigeon.UserProfilePigeon userProfile) {
        YandexMetrica.getReporter(context, apiKey).reportUserProfile(Converter.toNative(userProfile));
    }

    @Override
    public void reportRevenue(@NonNull String apiKey, @NonNull Pigeon.RevenuePigeon revenue) {
        YandexMetrica.getReporter(context, apiKey).reportRevenue(Converter.toNative(revenue));
    }

    @Override
    public void reportECommerce(@NonNull String apiKey, @NonNull Pigeon.ECommerceEventPigeon event) {
        final ECommerceEvent eCommerceEvent = ECommerceConverter.toNative(event);
        if (eCommerceEvent != null) {
            YandexMetrica.getReporter(context, apiKey).reportECommerce(eCommerceEvent);
        }
    }

    @Override
    public void reportAdRevenue(@NonNull String apiKey, @NonNull Pigeon.AdRevenuePigeon adRevenue) {
        YandexMetrica.getReporter(context, apiKey).reportAdRevenue(Converter.toNative(adRevenue));
    }
}
