/*
 * Version for Flutter
 * © 2023 YANDEX
 * You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * https://yandex.com/legal/appmetrica_sdk_agreement/
 */

#import "YMMFPigeon+Mapping.h"
#import "YMMFPigeonUtils.h"

@implementation YMMFAppMetricaConfigPigeon (Mapping)

+ (YMMFAppMetricaConfigPigeon *)fromMap:(NSDictionary *)dict
{
    if (dict == nil) {
        return nil;
    }
    YMMFAppMetricaConfigPigeon *pigeonResult = [[YMMFAppMetricaConfigPigeon alloc] init];
    pigeonResult.apiKey = [YMMFPigeonUtils getNullableObject:dict key:@"apiKey"];
    NSAssert(pigeonResult.apiKey != nil, @"");
    
    pigeonResult.appVersion = [YMMFPigeonUtils getNullableObject:dict key:@"appVersion"];
    pigeonResult.crashReporting = [YMMFPigeonUtils getNullableObject:dict key:@"crashReporting"];
    pigeonResult.firstActivationAsUpdate = [YMMFPigeonUtils getNullableObject:dict key:@"firstActivationAsUpdate"];
    pigeonResult.location = [YMMFLocationPigeon fromMap:[YMMFPigeonUtils getNullableObject:dict key:@"location"]];
    pigeonResult.locationTracking = [YMMFPigeonUtils getNullableObject:dict key:@"locationTracking"];
    pigeonResult.logs = [YMMFPigeonUtils getNullableObject:dict key:@"logs"];
    pigeonResult.sessionTimeout = [YMMFPigeonUtils getNullableObject:dict key:@"sessionTimeout"];
    pigeonResult.statisticsSending = [YMMFPigeonUtils getNullableObject:dict key:@"statisticsSending"];
    pigeonResult.preloadInfo = [YMMFPreloadInfoPigeon fromMap:[YMMFPigeonUtils getNullableObject:dict key:@"preloadInfo"]];
    pigeonResult.maxReportsInDatabaseCount = [YMMFPigeonUtils getNullableObject:dict key:@"maxReportsInDatabaseCount"];
    pigeonResult.nativeCrashReporting = [YMMFPigeonUtils getNullableObject:dict key:@"nativeCrashReporting"];
    pigeonResult.sessionsAutoTracking = [YMMFPigeonUtils getNullableObject:dict key:@"sessionsAutoTracking"];
    pigeonResult.errorEnvironment = [YMMFPigeonUtils getNullableObject:dict key:@"errorEnvironment"];
    pigeonResult.userProfileID = [YMMFPigeonUtils getNullableObject:dict key:@"userProfileID"];
    pigeonResult.revenueAutoTracking = [YMMFPigeonUtils getNullableObject:dict key:@"revenueAutoTracking"];
    pigeonResult.appOpenTrackingEnabled = [YMMFPigeonUtils getNullableObject:dict key:@"appOpenTrackingEnabled"];
    return pigeonResult;
}

- (NSDictionary *)toMap
{
    return @{
        @"apiKey" : (self.apiKey ?: [NSNull null]),
        @"appVersion" : (self.appVersion ?: [NSNull null]),
        @"crashReporting" : (self.crashReporting ?: [NSNull null]),
        @"firstActivationAsUpdate" : (self.firstActivationAsUpdate ?: [NSNull null]),
        @"location" : (self.location ? [self.location toMap] : [NSNull null]),
        @"locationTracking" : (self.locationTracking ?: [NSNull null]),
        @"logs" : (self.logs ?: [NSNull null]),
        @"sessionTimeout" : (self.sessionTimeout ?: [NSNull null]),
        @"statisticsSending" : (self.statisticsSending ?: [NSNull null]),
        @"preloadInfo" : (self.preloadInfo ? [self.preloadInfo toMap] : [NSNull null]),
        @"maxReportsInDatabaseCount" : (self.maxReportsInDatabaseCount ?: [NSNull null]),
        @"nativeCrashReporting" : (self.nativeCrashReporting ?: [NSNull null]),
        @"sessionsAutoTracking" : (self.sessionsAutoTracking ?: [NSNull null]),
        @"errorEnvironment" : (self.errorEnvironment ?: [NSNull null]),
        @"userProfileID" : (self.userProfileID ?: [NSNull null]),
        @"revenueAutoTracking" : (self.revenueAutoTracking ?: [NSNull null]),
        @"appOpenTrackingEnabled" : (self.appOpenTrackingEnabled ?: [NSNull null]),
    };
}

@end

@implementation YMMFLocationPigeon (Mapping)

+ (YMMFLocationPigeon *)fromMap:(NSDictionary *)dict
{
    if (dict == nil) {
        return nil;
    }
    YMMFLocationPigeon *pigeonResult = [[YMMFLocationPigeon alloc] init];
    pigeonResult.latitude = [YMMFPigeonUtils getNullableObject:dict key:@"latitude"];
    NSAssert(pigeonResult.latitude != nil, @"");
    pigeonResult.longitude = [YMMFPigeonUtils getNullableObject:dict key:@"longitude"];
    NSAssert(pigeonResult.longitude != nil, @"");
    pigeonResult.provider = [YMMFPigeonUtils getNullableObject:dict key:@"provider"];
    pigeonResult.altitude = [YMMFPigeonUtils getNullableObject:dict key:@"altitude"];
    pigeonResult.accuracy = [YMMFPigeonUtils getNullableObject:dict key:@"accuracy"];
    pigeonResult.course = [YMMFPigeonUtils getNullableObject:dict key:@"course"];
    pigeonResult.speed = [YMMFPigeonUtils getNullableObject:dict key:@"speed"];
    pigeonResult.timestamp = [YMMFPigeonUtils getNullableObject:dict key:@"timestamp"];
    return pigeonResult;
}

- (NSDictionary *)toMap
{
    return @{
        @"latitude" : (self.latitude ?: [NSNull null]),
        @"longitude" : (self.longitude ?: [NSNull null]),
        @"provider" : (self.provider ?: [NSNull null]),
        @"altitude" : (self.altitude ?: [NSNull null]),
        @"accuracy" : (self.accuracy ?: [NSNull null]),
        @"course" : (self.course ?: [NSNull null]),
        @"speed" : (self.speed ?: [NSNull null]),
        @"timestamp" : (self.timestamp ?: [NSNull null]),
    };
}

@end

@implementation YMMFPreloadInfoPigeon (Mapping)

+ (YMMFPreloadInfoPigeon *)fromMap:(NSDictionary *)dict
{
  if (dict == nil) {
    return nil;
  }
    YMMFPreloadInfoPigeon *pigeonResult = [[YMMFPreloadInfoPigeon alloc] init];
    pigeonResult.trackingId = [YMMFPigeonUtils getNullableObject:dict key:@"trackingId"];
    NSAssert(pigeonResult.trackingId != nil, @"");
    pigeonResult.additionalInfo = [YMMFPigeonUtils getNullableObject:dict key:@"additionalInfo"];
    return pigeonResult;
}

- (NSDictionary *)toMap
{
    return @{
        @"trackingId" : (self.trackingId ?: [NSNull null]),
        @"additionalInfo" : (self.additionalInfo ?: [NSNull null]),
    };
}

@end
